CREATE DATABASE scd;

CREATE TYPE public.operational_intent_state AS ENUM ('Unknown', 'Accepted', 'Activated', 'Nonconforming', 'Contingent');

CREATE TABLE scd.public.scd_subscriptions (
	id UUID NOT NULL,
	owner STRING NOT NULL,
	version INT4 NOT NULL DEFAULT 0:::INT8,
	url STRING NOT NULL,
	notification_index INT4 NULL DEFAULT 0:::INT8,
	notify_for_operations BOOL NULL DEFAULT false,
	notify_for_constraints BOOL NULL DEFAULT false,
	implicit BOOL NULL DEFAULT false,
	starts_at TIMESTAMPTZ NULL,
	ends_at TIMESTAMPTZ NULL,
	updated_at TIMESTAMPTZ NOT NULL,
	cells INT8[] NULL,
	CONSTRAINT "primary" PRIMARY KEY (id ASC),
	INDEX owner_idx (owner ASC),
	INDEX starts_at_idx (starts_at ASC),
	INDEX ends_at_idx (ends_at ASC),
	INVERTED INDEX cell_idx (cells),
	CONSTRAINT check_starts_at_ends_at_starts_at_ends_at CHECK (((starts_at IS NULL) OR (ends_at IS NULL)) OR (starts_at < ends_at)),
	CONSTRAINT check_notify_for_operations_notify_for_constraints CHECK (notify_for_operations OR notify_for_constraints)
);

CREATE TABLE scd.public.scd_operations (
	id UUID NOT NULL,
	owner STRING NOT NULL,
	version INT4 NOT NULL DEFAULT 0:::INT8,
	url STRING NOT NULL,
	altitude_lower FLOAT4 NULL,
	altitude_upper FLOAT4 NULL,
	starts_at TIMESTAMPTZ NULL,
	ends_at TIMESTAMPTZ NULL,
	subscription_id UUID NULL,
	updated_at TIMESTAMPTZ NOT NULL,
	state scd.public.operational_intent_state NOT NULL DEFAULT 'Unknown':::scd.public.operational_intent_state,
	cells INT8[] NULL,
	CONSTRAINT "primary" PRIMARY KEY (id ASC),
	CONSTRAINT fk_subscription_id_ref_scd_subscriptions FOREIGN KEY (subscription_id) REFERENCES public.scd_subscriptions(id) ON DELETE CASCADE,
	INDEX owner_idx (owner ASC),
	INDEX altitude_lower_idx (altitude_lower ASC),
	INDEX altitude_upper_idx (altitude_upper ASC),
	INDEX starts_at_idx (starts_at ASC),
	INDEX ends_at_idx (ends_at ASC),
	INDEX updated_at_idx (updated_at ASC),
	INDEX subscription_id_idx (subscription_id ASC),
	INVERTED INDEX cell_idx (cells),
	CONSTRAINT check_starts_at_ends_at_starts_at_ends_at CHECK (((starts_at IS NULL) OR (ends_at IS NULL)) OR (starts_at < ends_at))
);
